import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, ParamMap, Router } from '@angular/router';
import { Resource, IResource } from 'src/app/models/resource.model';
import { ResourceService } from 'src/app/services/resource.service';
import { FormBuilder, FormGroup, Validators, FormArray } from '@angular/forms';
import { filter, switchMap, mergeMap, merge, concatMap, map } from 'rxjs/operators';
import { Format } from 'src/app/models/format.model';
import { FormatService } from 'src/app/services';
import { from, forkJoin, Observable, of } from 'rxjs';

@Component({
  selector: 'app-resource-form',
  templateUrl: './resource-form.component.html',
  styleUrls: ['./resource-form.component.scss'],
})
export class ResourceFormComponent implements OnInit {

  resource: Resource = new Resource();
  formatsList: Format[] = [];
  form: FormGroup;
  title: string;

  constructor(
    private _resourceService: ResourceService,
    private _formatService: FormatService,
    private _route: ActivatedRoute,
    private _router: Router,
    private _fb: FormBuilder,
  ) {
  }

  ngOnInit() {
    this.title = this._route.snapshot.data.title;
    this.initForm();

    this._formatService.getAllFormats().subscribe((res) => {
      this.formatsList = res;
    });

    this._route.paramMap.pipe(
      filter((paramMap: ParamMap) => (paramMap.get('id') !== null)),
      switchMap((paramMap: ParamMap) => this._resourceService.findById(paramMap.get('id'))))
      .subscribe((resource: Resource) => {
        this.resource = resource;
        this.initForm();
      });

  }

  initForm() {
    const resourceFormats = new FormArray([]);
    this.resource.resourceFormats.forEach((resourceFormat) => {
      resourceFormats.push(this._fb.group({
        formatId: [resourceFormat.format.id, Validators.required],
        isProjectable: [resourceFormat.isProjectable, Validators.required],
        isCuttable: [resourceFormat.isCuttable, Validators.required],
        id: resourceFormat.id,
      }));
    });

    this.form = this._fb.group(
      {
        resourceFormats,
        id: [this.resource.id],
        name: [this.resource.name, Validators.required],
        acronym: [this.resource.acronym],
        type: [this.resource.type, Validators.required],
        isQueryable: [this.resource.isQueryable, Validators.required],
        isDownloadable: [this.resource.isDownloadable, Validators.required],
        isStandard: [this.resource.isStandard, Validators.required],
        description: [this.resource.description],
        parametersUrl: [this.resource.parametersUrl],
        messageWarning: [this.resource.messageWarning],
      });
  }

  onSubmit() {
    if (!this.formInvalid) {
      const resourceFormats = this.form.controls.resourceFormats['controls'].filter(e => e.dirty).map(e => e.value);
      const newResource = new Resource(this.form.value);
      let savedResource: Resource;
      this._resourceService.replaceOrCreate(newResource).pipe(
        mergeMap((resource) => {
          savedResource = resource;
          let actions: Observable<any>[] = [];

          const toAdd = resourceFormats.filter(e => !e.id);
          const toUpdate = resourceFormats.filter(e => e.id)
          const toDelete = this.resource.resourceFormats.map(e => e.id).filter((id) => {
            return !this.form.controls.resourceFormats['controls'].map(e => e.value.id).find(rfId => rfId === id);
          });

          if (toAdd.length > 0) {
            actions.push(this._resourceService.createResourceFormats(resource.id, toAdd));
          }

          if (toUpdate.length > 0) {
            actions.push(this._resourceService.updateResourceFormats(resource.id, toUpdate));
          }

          if (toDelete.length > 0) {
            actions.push(this._resourceService.deleteResourceFormats(resource.id, toDelete));
          }

          let res;
          (actions.length > 0) ? res = forkJoin(actions) : res = of(null);

          return res;
        }),
        map(() => savedResource.id),
      ).subscribe(
        (resourceCreatedId) => {
          this._router.navigate(['/resources', resourceCreatedId]);
        },
        (err) => {
        },
      );
    }
  }

  arrayToString(array: string[]) {
    let val = '';
    if (array && array.length) {
      val = array.join(',');
    }
    return val;
  }

  addResourceFormat() {
    // if (!this.form.controls.links) {
    //   this.organization.links = [];
    // }
    (this.form.controls.resourceFormats as FormArray).push(this._fb.group({
      formatId: [null, Validators.required],
      isProjectable: 0,
      isCuttable: 0,
    }));
  }

  removeResourceFormat(index) {
    (this.form.controls.resourceFormats as FormArray).removeAt(index);
  }

  compareFormats(format1: Format, format2: Format) {
    return format1 && format2 ? format1.id === format2.id : false;
  }

  // Getters for each property
  get name() {
    return this.form.controls['name'];
  }

  get type() {
    return this.form.controls['type'];
  }

  get isQueryable() {
    return this.form.controls['isQueryable'];
  }

  get description() {
    return this.form.controls['description'];
  }

  get formResourceFormats() {
    return this.form.controls.resourceFormats as FormArray;
  }

  // get outputFormats() {
  //   return this.form.controls['outputFormats'];
  // }

  get formInvalid() {
    return this.form.invalid;
  }
}
